@@ -21,11 +21,22 @@ import fnmatch
 import time
 import yumbased
 import shutil
+from  bz2 import BZ2File
+from urlgrabber import grabber
+import tempfile
+
+from yum import misc, Errors, to_unicode
+from yum.sqlutils import executeSQL
+from yum.packageSack import MetaSack
+from yum.packages import YumAvailablePackage
 
-from yum import misc, Errors
 import rpmUtils.transaction
 from utils import _, errorprint, MDError
 import readMetadata
+try:
+    import sqlite3 as sqlite
+except ImportError:
+    import sqlite
 
 try:
     import sqlitecachec
@@ -33,19 +44,20 @@ except ImportError:
     pass
 
 from utils import _gzipOpen, bzipFile, checkAndMakeDir, GzipFile, checksum_and_rename
+import deltarpms
 
-__version__ = '0.9.5'
+__version__ = '0.9.7'
 
 
 class MetaDataConfig(object):
     def __init__(self):
         self.quiet = False
         self.verbose = False
+        self.profile = False
         self.excludes = []
         self.baseurl = None
         self.groupfile = None
-        self.sumtype = 'sha'
-        self.noepoch = False # hmm - maybe a fixme?
+        self.sumtype = 'sha256'
         self.pretty = False
         self.cachedir = None 
         self.use_cache = False
@@ -53,6 +65,13 @@ class MetaDataConfig(object):
         self.checkts = False
         self.split = False        
         self.update = False
+        self.deltas = False # do the deltarpm thing
+        self.deltadir = None # where to put the .drpms - defaults to 'drpms' inside 'repodata'
+        self.delta_relative = 'drpms/'
+        self.oldpackage_paths = [] # where to look for the old packages - 
+        self.deltafile = 'prestodelta.xml.gz'
+        self.num_deltas = 1 # number of older versions to delta (max)
+        self.update_md_path = None 
         self.skip_stat = False
         self.database = False
         self.outputdir = None
@@ -60,6 +79,7 @@ class MetaDataConfig(object):
         self.dir_patterns = ['.*bin\/.*', '^\/etc\/.*']
         self.skip_symlinks = False
         self.pkglist = []
+        self.database_only = False
         self.primaryfile = 'primary.xml.gz'
         self.filelistsfile = 'filelists.xml.gz'
         self.otherfile = 'other.xml.gz'
@@ -72,7 +92,10 @@ class MetaDataConfig(object):
         self.directories = []
         self.changelog_limit = None # needs to be an int or None
         self.unique_md_filenames = False
-        
+        self.additional_metadata = {} # dict of 'type':'filename'
+        self.revision = str(int(time.time()))
+        self.content_tags = [] # flat list of strings (like web 2.0 tags)
+        self.distro_tags = []# [(cpeid(None allowed), human-readable-string)]
 
 class SimpleMDCallBack(object):
     def errorlog(self, thing):
@@ -100,8 +123,10 @@ class MetaDataGenerator:
                     
         self.ts = rpmUtils.transaction.initReadOnlyTransaction()
         self.pkgcount = 0
+        self.current_pkg = 0
         self.files = []
-        
+        self.rpmlib_reqs = {}
+                
         if not self.conf.directory and not self.conf.directories:
             raise MDError, "No directory given on which to run."
         
@@ -133,7 +158,6 @@ class MetaDataGenerator:
         if not self.conf.outputdir:
             self.conf.outputdir = os.path.join(self.conf.basedir, self.conf.relative_dir)
 
-
     def _test_setup_dirs(self):
         # start the sanity/stupidity checks
         for mydir in self.conf.directories:
@@ -162,22 +186,31 @@ class MetaDataGenerator:
         if not checkAndMakeDir(temp_final):
             raise MDError, _('Cannot create/verify %s') % temp_final
 
+        if self.conf.deltas:
+            temp_delta = os.path.join(self.conf.outputdir, self.conf.delta_relative)
+            if not checkAndMakeDir(temp_delta):
+                raise MDError, _('Cannot create/verify %s') % temp_delta
+            self.conf.deltadir = temp_delta
+
         if os.path.exists(os.path.join(self.conf.outputdir, self.conf.olddir)):
             raise MDError, _('Old data directory exists, please remove: %s') % self.conf.olddir
 
         # make sure we can write to where we want to write to:
         # and pickup the mdtimestamps while we're at it
-        for direc in ['tempdir', 'finaldir']:
-            for f in ['primaryfile', 'filelistsfile', 'otherfile', 'repomdfile']:
-                filepath = os.path.join(self.conf.outputdir, direc, f)
-                if os.path.exists(filepath):
-                    if not os.access(filepath, os.W_OK):
-                        raise MDError, _('error in must be able to write to metadata files:\n  -> %s') % filepath
+        direcs = ['tempdir' , 'finaldir']
+        if self.conf.deltas:
+            direcs.append('deltadir')
 
-                    if self.conf.checkts:
-                        timestamp = os.path.getctime(filepath)
-                        if timestamp > self.conf.mdtimestamp:
-                            self.conf.mdtimestamp = timestamp
+        for direc in direcs:
+            filepath = os.path.join(self.conf.outputdir, getattr(self.conf, direc))
+            if os.path.exists(filepath):
+                if not os.access(filepath, os.W_OK):
+                    raise MDError, _('error in must be able to write to metadata dir:\n  -> %s') % filepath
+
+                if self.conf.checkts:
+                    timestamp = os.path.getctime(filepath)
+                    if timestamp > self.conf.mdtimestamp:
+                        self.conf.mdtimestamp = timestamp
 
         if self.conf.groupfile:
             a = self.conf.groupfile
@@ -246,7 +279,8 @@ class MetaDataGenerator:
         """check the timestamp of our target dir. If it is not newer than the repodata
            return False, else True"""
         if self.conf.checkts:
-            files = self.getFileList(self.conf.basedir, self.conf.directory, '.rpm')
+            dn = os.path.join(self.conf.basedir, self.conf.directory)
+            files = self.getFileList(dn, '.rpm')
             files = self.trimRpms(files)
             for f in files:
                 fn = os.path.join(self.conf.basedir, self.conf.directory, f)
@@ -278,20 +312,33 @@ class MetaDataGenerator:
         # function for the first dir passed to --split, not all of them
         # this needs to be fixed by some magic in readMetadata.py
         # using opts.pkgdirs as a list, I think.
-        # FIXME - this needs to read the old repomd.xml to figure out where
-        # the files WERE to pass in the right fns.
         if self.conf.update:
             #build the paths
             opts = {
                 'verbose' : self.conf.verbose,
                 'pkgdir'  : os.path.normpath(self.package_dir)
             }
+
             if self.conf.skip_stat:
                 opts['do_stat'] = False
-                
+
+            if self.conf.update_md_path:
+                old_repo_path = os.path.normpath(self.conf.update_md_path)
+            else:
+                old_repo_path = self.conf.outputdir
+
             #and scan the old repo
-            self.oldData = readMetadata.MetadataIndex(self.conf.outputdir, opts)
-           
+            self.oldData = readMetadata.MetadataIndex(old_repo_path, opts)
+
+    def _setup_grabber(self):
+        if not hasattr(self, '_grabber'):
+            self._grabber = grabber.URLGrabber()
+    
+        return self._grabber
+
+    grabber = property(fget = lambda self: self._setup_grabber())
+    
+    
     def doPkgMetadata(self):
         """all the heavy lifting for the package metadata"""
         if self.conf.update:
@@ -301,8 +348,9 @@ class MetaDataGenerator:
             packages = self.conf.pkglist
         else:
             packages = self.getFileList(self.package_dir, '.rpm')
-            
-        packages = self.trimRpms(packages)
+        
+        if not isinstance(packages, MetaSack):
+            packages = self.trimRpms(packages)
         self.pkgcount = len(packages)
         self.openMetadataDocs()
         self.writeMetadataDocs(packages)
@@ -310,9 +358,14 @@ class MetaDataGenerator:
 
     # module
     def openMetadataDocs(self):
-        self.primaryfile = self._setupPrimary()
-        self.flfile = self._setupFilelists()
-        self.otherfile = self._setupOther()
+        if self.conf.database_only:
+            self.setup_sqlite_dbs()
+        else:
+            self.primaryfile = self._setupPrimary()
+            self.flfile = self._setupFilelists()
+            self.otherfile = self._setupOther()
+        if self.conf.deltas:
+            self.deltafile = self._setupDelta()
 
     def _setupPrimary(self):
         # setup the primary metadata file
@@ -340,78 +393,124 @@ class MetaDataGenerator:
         fo.write('<otherdata xmlns="http://linux.duke.edu/metadata/other" packages="%s">' %
                        self.pkgcount)
         return fo
+
+    def _setupDelta(self):
+        # setup the other file
+        deltafilepath = os.path.join(self.conf.outputdir, self.conf.tempdir, self.conf.deltafile)
+        fo = _gzipOpen(deltafilepath, 'w')
+        fo.write('<?xml version="1.0" encoding="UTF-8"?>\n')
+        fo.write('<prestodelta>\n')
+        return fo
         
 
-    def read_in_package(self, rpmfile, pkgpath=None):
+    def read_in_package(self, rpmfile, pkgpath=None, reldir=None):
         """rpmfile == relative path to file from self.packge_dir"""
-        # TODO/FIXME
-        # consider adding a routine to download the package from a remote location
-        # to a tempdir, operate on it, then use that location as a the baseurl
-        # for the package. That would make it possible to have repos entirely 
-        # comprised of remote packages.
+        remote_package = False
+        baseurl = self.conf.baseurl
 
         if not pkgpath:
             pkgpath = self.package_dir
 
-        rpmfile = '%s/%s' % (pkgpath, rpmfile)
+        if not rpmfile.strip():
+            raise MDError, "Blank filename passed in, skipping"
+            
+        if rpmfile.find("://") != -1:
+            remote_package = True
+            
+            if not hasattr(self, 'tempdir'):
+                self.tempdir = tempfile.mkdtemp()
+                
+            pkgname = os.path.basename(rpmfile)
+            baseurl = os.path.dirname(rpmfile)
+            reldir = self.tempdir       
+            dest = os.path.join(self.tempdir, pkgname)
+            if not self.conf.quiet:
+                self.callback.log('\nDownloading %s' % rpmfile)                        
+            try:
+                rpmfile = self.grabber.urlgrab(rpmfile, dest)
+            except grabber.URLGrabError, e:
+                raise MDError, "Unable to retrieve remote package %s: %s" %(rpmfile, e)
+
+            
+        else:
+            rpmfile = '%s/%s' % (pkgpath, rpmfile)
+            
         try:
             po = yumbased.CreateRepoPackage(self.ts, rpmfile)
         except Errors.MiscError, e:
             raise MDError, "Unable to open package: %s" % e
-        # if we're going to add anything in from outside, here is where
-        # you can do it
-        po.crp_changelog_limit = self.conf.changelog_limit
-        po.crp_cachedir = self.conf.cachedir
-
-        # FIXME if we wanted to put in a baseurl-per-package here is where 
-        # we should do it
-        # it would be easy to have a lookup dict in the MetaDataConfig object
-        # and work down from there for the baseurl
-
+        # external info we need
+        po._cachedir = self.conf.cachedir
+        po._baseurl = baseurl
+        po._reldir = reldir
+        po._packagenumber = self.current_pkg
+        for r in po.requires_print:
+            if r.startswith('rpmlib('):
+                self.rpmlib_reqs[r] = 1
+           
         if po.checksum in (None, ""):
-            raise MDError, "No Package ID found for package %s, not going to add it" % e
+            raise MDError, "No Package ID found for package %s, not going to add it" % po
         
         return po
 
-    def writeMetadataDocs(self, pkglist=[], pkgpath=None, current=0):
+    def writeMetadataDocs(self, pkglist=[], pkgpath=None):
 
         if not pkglist:
             pkglist = self.conf.pkglist           
+
         if not pkgpath:
             directory=self.conf.directory
         else:
             directory=pkgpath
 
         for pkg in pkglist:
-            current+=1
+            self.current_pkg += 1
             recycled = False
-
+            
             # look to see if we can get the data from the old repodata
             # if so write this one out that way
             if self.conf.update:
                 #see if we can pull the nodes from the old repo
                 #print self.oldData.basenodes.keys()
-                nodes = self.oldData.getNodes(pkg)
+                old_pkg = pkg
+                if pkg.find("://") != -1:
+                    old_pkg = os.path.basename(pkg)
+                nodes = self.oldData.getNodes(old_pkg)
                 if nodes is not None:
                     recycled = True
-
+                
+                # FIXME also open up the delta file
             
             # otherwise do it individually
             if not recycled:
                 #scan rpm files
-                try:
-                    po = self.read_in_package(pkg, pkgpath=pkgpath)
-                except MDError, e:
-                    # need to say something here
-                    self.callback.errorlog("\nError %s: %s\n" % (pkg, e))
-                    continue
-                if not pkgpath:
+                if not pkgpath:   
                     reldir = os.path.join(self.conf.basedir, directory)
                 else:
                     reldir = pkgpath
-                self.primaryfile.write(po.do_primary_xml_dump(reldir, baseurl=self.conf.baseurl))
-                self.flfile.write(po.do_filelists_xml_dump())
-                self.otherfile.write(po.do_other_xml_dump())
+                
+                if not isinstance(pkg, YumAvailablePackage):
+
+                    try:
+                        po = self.read_in_package(pkg, pkgpath=pkgpath, reldir=reldir)
+                    except MDError, e:
+                        # need to say something here
+                        self.callback.errorlog("\nError %s: %s\n" % (pkg, e))
+                        continue
+                    # we can use deltas:
+                    self._do_delta_rpm_package(po)
+
+                else:
+                    po = pkg
+
+                if self.conf.database_only:
+                    pass # disabled right now for sanity reasons (mine)
+                    #po.do_sqlite_dump(self.md_sqlite)
+                else:
+                    self.primaryfile.write(po.xml_dump_primary_metadata())
+                    self.flfile.write(po.xml_dump_filelists_metadata())
+                    self.otherfile.write(po.xml_dump_other_metadata(
+                              clog_limit=self.conf.changelog_limit))
             else:
                 if self.conf.verbose:
                     self.callback.log(_("Using data from old metadata for %s") % pkg)
@@ -431,37 +530,139 @@ class MetaDataGenerator:
                     outfile.write('\n')
 
                 self.oldData.freeNodes(pkg)
+                #FIXME - if we're in update and we have deltas enabled
+                #        check the presto data for this pkg and write its info back out
+                #       to our deltafile
 
             if not self.conf.quiet:
                 if self.conf.verbose:
-                    self.callback.log('%d/%d - %s' % (current, self.pkgcount, pkg))
+                    self.callback.log('%d/%d - %s' % (self.current_pkg, self.pkgcount, pkg))
                 else:
-                    self.callback.progress(pkg, current, self.pkgcount)
+                    self.callback.progress(pkg, self.current_pkg, self.pkgcount)
 
-        return current
+        return self.current_pkg
 
 
     def closeMetadataDocs(self):
         if not self.conf.quiet:
             self.callback.log('')
 
+        
         # save them up to the tmp locations:
         if not self.conf.quiet:
             self.callback.log(_('Saving Primary metadata'))
-        self.primaryfile.write('\n</metadata>')
-        self.primaryfile.close()
+        if self.conf.database_only:
+            self.md_sqlite.pri_cx.close()
+        else:
+            self.primaryfile.write('\n</metadata>')
+            self.primaryfile.close()
 
         if not self.conf.quiet:
             self.callback.log(_('Saving file lists metadata'))
-        self.flfile.write('\n</filelists>')
-        self.flfile.close()
+        if self.conf.database_only:
+            self.md_sqlite.file_cx.close()
+        else:
+            self.flfile.write('\n</filelists>')
+            self.flfile.close()
 
         if not self.conf.quiet:
             self.callback.log(_('Saving other metadata'))
-        self.otherfile.write('\n</otherdata>')
-        self.otherfile.close()
+        if self.conf.database_only:
+            self.md_sqlite.other_cx.close()
+        else:
+            self.otherfile.write('\n</otherdata>')
+            self.otherfile.close()
 
-    def addArbitraryMetadata(self, mdfile, mdtype, xml_node, compress=True):
+        if self.conf.deltas:
+            if not self.conf.quiet:
+                self.callback.log(_('Saving delta metadata'))
+            self.deltafile.write(self.generate_delta_xml())
+            self.deltafile.write('\n</prestodelta>')
+            self.deltafile.close()
+
+    def _do_delta_rpm_package(self, pkg):
+        """makes the drpms, if possible, for this package object.
+           returns the presto/delta xml metadata as a string
+        """
+
+        # generate a list of all the potential 'old rpms'
+        opd = self._get_old_package_dict() # yes I could make this a property but <shrug>
+        
+        # for each of our old_package_paths - make a drpm from the newest of that pkg
+        # get list of potential candidates which are likely to match
+        for d in self.conf.oldpackage_paths:
+            pot_cand = []
+            for fn in opd[d]:
+                if os.path.basename(fn).startswith(pkg.name):
+                    pot_cand.append(fn)
+            
+            candidates = []
+            for fn in pot_cand:
+                try:
+                    thispo = yumbased.CreateRepoPackage(self.ts, fn)
+                except Errors.MiscError, e:
+                    continue
+                if (thispo.name, thispo.arch) != (pkg.name, pkg.arch):
+                    # not the same, doesn't matter
+                    continue
+                if thispo == pkg: #exactly the same, doesn't matter
+                    continue
+                if thispo.EVR >= pkg.EVR: # greater or equal, doesn't matter
+                    continue
+                candidates.append(thispo)
+                candidates.sort()
+                candidates.reverse()
+
+            for delta_p in candidates[0:self.conf.num_deltas]:
+                #make drpm of pkg and delta_p
+                drpmfn = deltarpms.create_drpm(delta_p, pkg, self.conf.deltadir)
+                self.callback.log('created drpm from %s to %s: %s' % (
+                        delta_p, pkg, drpmfn))
+
+    def _get_old_package_dict(self):
+        if hasattr(self, '_old_package_dict'):
+            return self._old_package_dict
+        
+        self._old_package_dict = {}
+        opl = []
+        for d in self.conf.oldpackage_paths:
+            for f in self.getFileList(d, 'rpm'):
+                if not self._old_package_dict.has_key(d):
+                    self._old_package_dict[d] = []
+                self._old_package_dict[d].append(d + '/' + f)
+                    
+        return self._old_package_dict
+
+    def generate_delta_xml(self):
+        """take the delta rpm output dir, process all the drpm files
+           produce the text output for the presto/delta xml metadata"""
+        # go through the drpm dir
+        # for each file -store the drpm info in a dict based on its target. Just
+        # appending the output. for each of the keys in the dict, return
+        # the tag for the target + each of the drpm infos + closure for the target
+        # tag
+        targets = {}
+        result = u''
+        for drpm_fn in self.getFileList(self.conf.deltadir, 'drpm'):
+            drpm_rel_fn = os.path.normpath(self.conf.delta_relative + '/' + drpm_fn) # this is annoying
+            drpm_po = yumbased.CreateRepoPackage(self.ts, self.conf.deltadir + '/' + drpm_fn)
+            
+            drpm = deltarpms.DeltaRPMPackage(drpm_po, self.conf.outputdir, drpm_rel_fn)
+            if not targets.has_key(drpm_po.pkgtup):
+                targets[drpm_po.pkgtup] = u''
+            targets[drpm_po.pkgtup] += drpm.xml_dump_metadata()
+        
+        for (n,e,v,r,a) in targets.keys():
+            result += """  <newpackage name="%s" epoch="%s" version="%s" release="%s" arch="%s"\n""" % (
+                    n,e,v,r,a)
+            for src in targets[(n,e,v,r,a)]:
+                result += src
+            result += """   </newpackage>\n"""
+
+        return result
+
+    def addArbitraryMetadata(self, mdfile, mdtype, xml_node, compress=True, 
+                                             compress_type='gzip', attribs={}):
         """add random metadata to the repodata dir and repomd.xml
            mdfile = complete path to file
            mdtype = the metadata type to use
@@ -474,9 +675,14 @@ class MetaDataGenerator:
         fo = open(mdfile, 'r')
         outdir = os.path.join(self.conf.outputdir, self.conf.tempdir)
         if compress:
-            sfile = '%s.gz' % sfile
-            outfn = os.path.join(outdir, sfile)
-            output = GzipFile(filename = outfn, mode='wb')
+            if compress_type == 'gzip':
+                sfile = '%s.gz' % sfile
+                outfn = os.path.join(outdir, sfile)
+                output = GzipFile(filename = outfn, mode='wb')
+            elif compress_type == 'bzip2':
+                sfile = '%s.bz2' % sfile
+                outfn = os.path.join(outdir, sfile)
+                output = BZ2File(filename = outfn, mode='wb')
         else:
             outfn  = os.path.join(outdir, sfile)
             output = open(outfn, 'w')
@@ -513,6 +719,10 @@ class MetaDataGenerator:
             opencsum.newProp('type', self.conf.sumtype)
 
         timestamp = data.newChild(None, 'timestamp', str(timest))
+
+        # add the random stuff
+        for (k,v) in attribs.items():
+            data.newChild(None, k, str(v))
            
             
     def doRepoMetadata(self):
@@ -521,15 +731,36 @@ class MetaDataGenerator:
         reporoot = repodoc.newChild(None, "repomd", None)
         repons = reporoot.newNs('http://linux.duke.edu/metadata/repo', None)
         reporoot.setNs(repons)
+        rpmns = reporoot.newNs("http://linux.duke.edu/metadata/rpm", 'rpm')        
         repopath = os.path.join(self.conf.outputdir, self.conf.tempdir)
         repofilepath = os.path.join(repopath, self.conf.repomdfile)
+        
+        revision = reporoot.newChild(None, 'revision', self.conf.revision)
+        if self.conf.content_tags or self.conf.distro_tags:
+            tags = reporoot.newChild(None, 'tags', None)
+            for item in self.conf.content_tags:
+                c_tags = tags.newChild(None, 'content', item)
+            for (cpeid,item) in self.conf.distro_tags:
+                d_tags = tags.newChild(None, 'distro', item)
+                if cpeid:
+                    d_tags.newProp('cpeid', cpeid)
 
         sumtype = self.conf.sumtype
-        workfiles = [(self.conf.otherfile, 'other',),
-                     (self.conf.filelistsfile, 'filelists'),
-                     (self.conf.primaryfile, 'primary')]
-        repoid='garbageid'
+        if self.conf.database_only:
+            workfiles = []
+            db_workfiles = [(self.md_sqlite.pri_sqlite_file, 'primary_db'),
+                            (self.md_sqlite.file_sqlite_file, 'filelists_db'),
+                            (self.md_sqlite.other_sqlite_file, 'other_db')]
+            dbversion = '10'                            
+        else:
+            workfiles = [(self.conf.otherfile, 'other',),
+                         (self.conf.filelistsfile, 'filelists'),
+                         (self.conf.primaryfile, 'primary')]
+            db_workfiles = []
+            repoid='garbageid'
         
+        if self.conf.deltas:
+            workfiles.append((self.conf.deltafile, 'deltainfo'))
         if self.conf.database:
             if not self.conf.quiet: self.callback.log('Generating sqlite DBs')
             try:
@@ -640,11 +871,31 @@ class MetaDataGenerator:
 
         if not self.conf.quiet and self.conf.database: self.callback.log('Sqlite DBs complete')        
 
+        for (fn, ftype) in db_workfiles:
+            attribs = {'database_version':dbversion}
+            self.addArbitraryMetadata(fn, ftype, reporoot, compress=True, 
+                                      compress_type='bzip2', attribs=attribs)
+            try:
+                os.unlink(fn)
+            except (IOError, OSError), e:
+                pass
 
+            
         if self.conf.groupfile is not None:
             self.addArbitraryMetadata(self.conf.groupfile, 'group_gz', reporoot)
             self.addArbitraryMetadata(self.conf.groupfile, 'group', reporoot, compress=False)            
+        
+        if self.conf.additional_metadata:
+            for md_type, mdfile in self.conf.additional_metadata.items():
+                self.addArbitraryMetadata(mdfile, md_type, reporoot)
 
+        # FIXME - disabled until we decide how best to use this
+        #if self.rpmlib_reqs:
+        #    rpmlib = reporoot.newChild(rpmns, 'lib', None)
+        #    for r in self.rpmlib_reqs.keys():
+        #        req  = rpmlib.newChild(rpmns, 'requires', r)
+                
+            
         # save it down
         try:
             repodoc.saveFormatFileEnc(repofilepath, 'UTF-8', 1)
@@ -729,6 +980,16 @@ class MetaDataGenerator:
             self.errorlog(_('Error was %s') % e)
             self.errorlog(_('Please clean up this directory manually.'))
 
+    def setup_sqlite_dbs(self, initdb=True):
+        """sets up the sqlite dbs w/table schemas and db_infos"""
+        destdir = os.path.join(self.conf.outputdir, self.conf.tempdir)
+        try:
+            self.md_sqlite = MetaDataSqlite(destdir)
+        except sqlite.OperationalError, e:
+            raise MDError, _('Cannot create sqlite databases: %s.\nMaybe you need to clean up a .repodata dir?') % e
+        
+    
+    
 class SplitMetaDataGenerator(MetaDataGenerator):
     """takes a series of dirs and creates repodata for all of them
        most commonly used with -u media:// - if no outputdir is specified
@@ -787,17 +1048,107 @@ class SplitMetaDataGenerator(MetaDataGenerator):
             self.pkgcount += len(filematrix[mydir])
 
         mediano = 1
-        current = 0
+        self.current_pkg = 0
         self.conf.baseurl = self._getFragmentUrl(self.conf.baseurl, mediano)
         self.openMetadataDocs()
         original_basedir = self.conf.basedir
         for mydir in self.conf.directories:
             self.conf.baseurl = self._getFragmentUrl(self.conf.baseurl, mediano)
-            current = self.writeMetadataDocs(filematrix[mydir], mydir, current)
+            self.writeMetadataDocs(filematrix[mydir], mydir)
             mediano += 1
         self.conf.baseurl = self._getFragmentUrl(self.conf.baseurl, 1)
         self.closeMetadataDocs()
 
 
 
+class MetaDataSqlite(object):
+    def __init__(self, destdir):
+        self.pri_sqlite_file = os.path.join(destdir, 'primary.sqlite')
+        self.pri_cx = sqlite.Connection(self.pri_sqlite_file)
+        self.file_sqlite_file = os.path.join(destdir, 'filelists.sqlite')
+        self.file_cx = sqlite.Connection(self.file_sqlite_file)
+        self.other_sqlite_file = os.path.join(destdir, 'other.sqlite')
+        self.other_cx = sqlite.Connection(self.other_sqlite_file)
+        self.primary_cursor = self.pri_cx.cursor()
+
+        self.filelists_cursor = self.file_cx.cursor()
+        
+        self.other_cursor = self.other_cx.cursor()
+                
+        self.create_primary_db()
+        self.create_filelists_db()
+        self.create_other_db()
+        
+    def create_primary_db(self):
+        # make the tables
+        schema = [
+        """PRAGMA synchronous="OFF";""",
+        """pragma locking_mode="EXCLUSIVE";""",
+        """CREATE TABLE conflicts (  name TEXT,  flags TEXT,  epoch TEXT,  version TEXT,  release TEXT,  pkgKey INTEGER );""",
+        """CREATE TABLE db_info (dbversion INTEGER, checksum TEXT);""",
+        """CREATE TABLE files (  name TEXT,  type TEXT,  pkgKey INTEGER);""",
+        """CREATE TABLE obsoletes (  name TEXT,  flags TEXT,  epoch TEXT,  version TEXT,  release TEXT,  pkgKey INTEGER );""",
+        """CREATE TABLE packages (  pkgKey INTEGER PRIMARY KEY,  pkgId TEXT,  name TEXT,  arch TEXT,  version TEXT,  epoch TEXT,  release TEXT,  summary TEXT,  description TEXT,  url TEXT,  time_file INTEGER,  time_build INTEGER,  rpm_license TEXT,  rpm_vendor TEXT,  rpm_group TEXT,  rpm_buildhost TEXT,  rpm_sourcerpm TEXT,  rpm_header_start INTEGER,  rpm_header_end INTEGER,  rpm_packager TEXT,  size_package INTEGER,  size_installed INTEGER,  size_archive INTEGER,  location_href TEXT,  location_base TEXT,  checksum_type TEXT);""",
+        """CREATE TABLE provides (  name TEXT,  flags TEXT,  epoch TEXT,  version TEXT,  release TEXT,  pkgKey INTEGER );""",
+        """CREATE TABLE requires (  name TEXT,  flags TEXT,  epoch TEXT,  version TEXT,  release TEXT,  pkgKey INTEGER , pre BOOL DEFAULT FALSE);""",
+        """CREATE INDEX filenames ON files (name);""",
+        """CREATE INDEX packageId ON packages (pkgId);""",
+        """CREATE INDEX packagename ON packages (name);""",
+        """CREATE INDEX pkgconflicts on conflicts (pkgKey);""",
+        """CREATE INDEX pkgobsoletes on obsoletes (pkgKey);""",
+        """CREATE INDEX pkgprovides on provides (pkgKey);""",
+        """CREATE INDEX pkgrequires on requires (pkgKey);""",
+        """CREATE INDEX providesname ON provides (name);""",
+        """CREATE INDEX requiresname ON requires (name);""",
+        """CREATE TRIGGER removals AFTER DELETE ON packages  
+             BEGIN    
+             DELETE FROM files WHERE pkgKey = old.pkgKey;    
+             DELETE FROM requires WHERE pkgKey = old.pkgKey;    
+             DELETE FROM provides WHERE pkgKey = old.pkgKey;    
+             DELETE FROM conflicts WHERE pkgKey = old.pkgKey;    
+             DELETE FROM obsoletes WHERE pkgKey = old.pkgKey;
+             END;""",
+         """INSERT into db_info values (%s, 'direct_create');""" % sqlitecachec.DBVERSION,
+             ]
+        
+        for cmd in schema:
+            executeSQL(self.primary_cursor, cmd)
+
+    def create_filelists_db(self):
+        schema = [
+            """PRAGMA synchronous="0FF";""",
+            """pragma locking_mode="EXCLUSIVE";""",
+            """CREATE TABLE db_info (dbversion INTEGER, checksum TEXT);""",
+            """CREATE TABLE filelist (  pkgKey INTEGER,  dirname TEXT,  filenames TEXT,  filetypes TEXT);""",
+            """CREATE TABLE packages (  pkgKey INTEGER PRIMARY KEY,  pkgId TEXT);""",
+            """CREATE INDEX dirnames ON filelist (dirname);""",
+            """CREATE INDEX keyfile ON filelist (pkgKey);""",
+            """CREATE INDEX pkgId ON packages (pkgId);""",
+            """CREATE TRIGGER remove_filelist AFTER DELETE ON packages  
+                   BEGIN    
+                   DELETE FROM filelist WHERE pkgKey = old.pkgKey;  
+                   END;""",
+         """INSERT into db_info values (%s, 'direct_create');""" % sqlitecachec.DBVERSION,                   
+            ]
+        for cmd in schema:
+            executeSQL(self.filelists_cursor, cmd)
+        
+    def create_other_db(self):
+        schema = [
+            """PRAGMA synchronous="OFF";""",
+            """pragma locking_mode="EXCLUSIVE";""",
+            """CREATE TABLE changelog (  pkgKey INTEGER,  author TEXT,  date INTEGER,  changelog TEXT);""",
+            """CREATE TABLE db_info (dbversion INTEGER, checksum TEXT);""",
+            """CREATE TABLE packages (  pkgKey INTEGER PRIMARY KEY,  pkgId TEXT);""",
+            """CREATE INDEX keychange ON changelog (pkgKey);""",
+            """CREATE INDEX pkgId ON packages (pkgId);""",
+            """CREATE TRIGGER remove_changelogs AFTER DELETE ON packages  
+                 BEGIN    
+                 DELETE FROM changelog WHERE pkgKey = old.pkgKey;  
+                 END;""",
+         """INSERT into db_info values (%s, 'direct_create');""" % sqlitecachec.DBVERSION,                 
+            ]
+            
+        for cmd in schema:
+            executeSQL(self.other_cursor, cmd)
 
