@@ -20,7 +20,9 @@
 
 import os
 import sys
+import re
 from optparse import OptionParser
+import time
 
 import createrepo
 from createrepo import MDError
@@ -39,6 +41,8 @@ def parseArgs(args, conf):
                       help="output nothing except for serious errors")
     parser.add_option("-v", "--verbose", default=False, action="store_true",
                       help="output more debugging info.")
+    parser.add_option("--profile", default=False, action="store_true",
+                      help="output timing/profile info.")
     parser.add_option("-x", "--excludes", default=[], action="append",
                       help="files to exclude")
     parser.add_option("--basedir", default=os.getcwd(),
@@ -47,12 +51,8 @@ def parseArgs(args, conf):
                       help="baseurl to append on all files")
     parser.add_option("-g", "--groupfile", default=None,
                       help="path to groupfile to include in metadata")
-    parser.add_option("-s", "--checksum", default="sha", dest='sumtype',
+    parser.add_option("-s", "--checksum", default="sha256", dest='sumtype',
                       help="Deprecated, ignore")
-    parser.add_option("-n", "--noepoch", default=False, action="store_true",
-                      help="don't add zero epochs for non-existent epochs"\
-                         "(incompatible with yum and smart but required for" \
-                         "systems with rpm < 4.2.1)")
     parser.add_option("-p", "--pretty", default=False, action="store_true",
                       help="make sure all xml generated is formatted")
     parser.add_option("-c", "--cachedir", default=None,
@@ -61,8 +61,13 @@ def parseArgs(args, conf):
       help="check timestamps on files vs the metadata to see if we need to update")
     parser.add_option("-d", "--database", default=False, action="store_true",
                       help="create sqlite database files")
+    # temporarily disabled
+    #parser.add_option("--database-only", default=False, action="store_true", dest='database_only',
+    #                  help="Only make the sqlite databases - does not work with --update, yet")
     parser.add_option("--update", default=False, action="store_true",
                       help="use the existing repodata to speed up creation of new")
+    parser.add_option("--update-md-path", default=None, dest='update_md_path',
+                      help="use the existing repodata  for --update from this path")
     parser.add_option("--skip-stat", dest='skip_stat', default=False, action="store_true",
                       help="skip the stat() call on a --update, assumes if the file" \
                             "name is the same then the file is still the same" \
@@ -82,8 +87,21 @@ def parseArgs(args, conf):
                       default=False, action="store_true",
                       help="include the file's checksum in the filename, helps" \
                            "with proxies")
-                           
-    (opts, argsleft) = parser.parse_args()
+    parser.add_option("--distro", default=[], action="append",
+                      help="distro tag and optional cpeid: --distro 'cpeid,textname'")
+    parser.add_option("--content", default=[], dest='content_tags', action="append",
+                      help="tags for the content in the repository")
+    parser.add_option("--revision", default=None,
+                      help="user-specified revision for this repository")
+    parser.add_option("--deltas", default=False, action="store_true",
+                      help="create delta rpms and metadata")
+    parser.add_option("--oldpackagedirs", default=[], dest="oldpackage_paths", 
+                      action="append", help="paths to look for older pkgs to delta against")
+    parser.add_option("--num-deltas", default=1, dest='num_deltas', type='int',
+                      help="the number of older versions to make deltas against")
+
+
+    (opts, argsleft) = parser.parse_args(args)
     if len(argsleft) > 1 and not opts.split:
         errorprint(_('Error: Only one directory allowed per run.'))
         parser.print_usage()
@@ -114,11 +132,22 @@ def parseArgs(args, conf):
     conf.directory = directory
     conf.directories = directories
 
+    # distro tag parsing
+
+    for spec in opts.distro:
+        if spec.find(',') == -1:
+            conf.distro_tags.append((None,spec))
+        else:
+            splitspec = spec.split(',')
+            conf.distro_tags.append((splitspec[0], splitspec[1]))
+
     lst = []
     if conf.pkglist:
         pfo = open(conf.pkglist, 'r')
         for line in pfo.readlines():
-            line = line.replace('\n', '')
+            line = line.strip()
+            if re.match('^\s*\#.*', line) or re.match('^\s*$', line):
+                continue
             lst.append(line)
         pfo.close()
             
@@ -137,14 +166,19 @@ class MDCallBack(object):
         print thing
     
     def progress(self, item, current, total):
-        sys.stdout.write('\r' + ' ' * 80)
-        sys.stdout.write("\r%d/%d - %s" % (current, total, item))
+        beg = "%*d/%d - " % (len(str(total)), current, total)
+        left = 80 - len(beg)
+        sys.stdout.write("\r%s%-*.*s" % (beg, left, left, item))
         sys.stdout.flush()
         
 def main(args):
+    start_st = time.time()
     conf = createrepo.MetaDataConfig()
     conf = parseArgs(args, conf)
+    if conf.profile:
+        print ('start time: %0.3f' % (time.time() - start_st))
 
+    mid_st = time.time()       
     try:
         if conf.split:
             mdgen = createrepo.SplitMetaDataGenerator(config_obj=conf, callback=MDCallBack())
@@ -155,9 +189,22 @@ def main(args):
                     print _('repo is up to date')
                 sys.exit(0)
 
+        if conf.profile:
+            print ('mid time: %0.3f' % (time.time() - mid_st))
+                
+        pm_st = time.time()
         mdgen.doPkgMetadata()
+        if conf.profile:
+            print ('pm time: %0.3f' % (time.time() - pm_st))
+        rm_st = time.time()
         mdgen.doRepoMetadata()
+        if conf.profile:
+            print ('rm time: %0.3f' % (time.time() - rm_st))
+        fm_st = time.time()       
         mdgen.doFinalMove()
+        if conf.profile:
+            print ('fm time: %0.3f' % (time.time() - fm_st))
+        
         
     except MDError, e:
         errorprint(_('%s') % e)
