@@ -35,6 +35,7 @@ class RepoMetadata:
         """ Parses the repomd.xml file existing in the given repo directory. """
         self.repodir = os.path.abspath(repo)
         self.repomdxml = os.path.join(self.repodir, 'repomd.xml')
+        self.checksum_type = 'sha256'
         if not os.path.exists(self.repomdxml):
             raise Exception('%s not found' % self.repomdxml)
         self.doc = minidom.parse(self.repomdxml)
@@ -62,7 +63,10 @@ class RepoMetadata:
             mdname = 'updateinfo.xml'
         elif isinstance(metadata, str):
             if os.path.exists(metadata):
-                oldmd = file(metadata, 'r')
+                if metadata.endswith('.gz'):
+                    oldmd = GzipFile(filename=metadata, mode='rb')
+                else:
+                    oldmd = file(metadata, 'r')
                 md = oldmd.read()
                 oldmd.close()
                 mdname = os.path.basename(metadata)
@@ -72,7 +76,8 @@ class RepoMetadata:
             raise Exception('invalid metadata type')
 
         ## Compress the metadata and move it into the repodata
-        mdname += '.gz'
+        if not mdname.endswith('.gz'):
+            mdname += '.gz'
         mdtype = mdname.split('.')[0]
         destmd = os.path.join(self.repodir, mdname)
         newmd = GzipFile(filename=destmd, mode='wb')
@@ -80,7 +85,7 @@ class RepoMetadata:
         newmd.close()
         print "Wrote:", destmd
 
-        open_csum = checksum('sha', metadata)
+        open_csum = checksum(self.checksum_type, metadata)
 
 
         csum, destmd = checksum_and_rename(destmd)
@@ -101,13 +106,13 @@ class RepoMetadata:
         self._insert_element(data, 'location',
                              attrs={ 'href' : 'repodata/' + base_destmd })
         data.appendChild(self.doc.createTextNode("\n    "))
-        self._insert_element(data, 'checksum', attrs={ 'type' : 'sha' },
+        self._insert_element(data, 'checksum', attrs={ 'type' : self.checksum_type },
                              text=csum)
         data.appendChild(self.doc.createTextNode("\n    "))
         self._insert_element(data, 'timestamp',
                              text=str(os.stat(destmd).st_mtime))
         data.appendChild(self.doc.createTextNode("\n    "))
-        self._insert_element(data, 'open-checksum', attrs={ 'type' : 'sha' },
+        self._insert_element(data, 'open-checksum', attrs={ 'type' : self.checksum_type },
                              text=open_csum)
 
         data.appendChild(self.doc.createTextNode("\n  "))
@@ -131,6 +136,14 @@ if __name__ == '__main__':
     if len(sys.argv) != 3 or '-h' in sys.argv:
         print "Usage: %s <input metadata> <output repodata>" % sys.argv[0]
         sys.exit()
+    try:
+        repomd = RepoMetadata(sys.argv[2])
+    except Exception, e:
+        print "Could not access repository: %s" % str(e)
+        sys.exit(1)
+    try:
+        repomd.add(sys.argv[1])
+    except Exception, e:
+        print "Could not add metadata from file %s: %s" % (sys.argv[1], str(e))
+        sys.exit(1)
 
-    repomd = RepoMetadata(sys.argv[2])
-    repomd.add(sys.argv[1])
